from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.db.models import Q
from django.shortcuts import redirect, render

from app.articles.models import Article
from app.feeds.models import Feed
from app.questions.models import Question


@login_required
def search(request):
    if 'q' in request.GET:
        querystring = request.GET.get('q').strip()
        if len(querystring) == 0:
            return redirect('/search/')

        try:
            search_type = request.GET.get('type')
            if search_type not in ['feed', 'articles', 'questions', 'users']:
                search_type = 'feed'

        except Exception:
            search_type = 'feed'

        count = {}
        results = {}

        kinds = {
            Feed: ['post'],
            Article: ['title', 'content'],
            Question: ['title', 'description'],
            User: ['username', 'first_name', 'last_name'],
            }

        for kind, terms in kinds.iteritems():
            clause = " OR ".join(
                ["%s LIKE '%%%s%%'" % (term, querystring) 
                     for term in terms])
            
            if kind == Feed:
                bucket = "feed" # xxx 
            else:
                bucket = kind.__name__.lower() + "s"
            results[bucket] = kind.objects.extra(where=[clause])
            
        count['feed'] = results['feed'].count()
        count['articles'] = results['articles'].count()
        count['questions'] = results['questions'].count()
        count['users'] = results['users'].count()

        return render(request, 'search/results.html', {
            'hide_search': True,
            'querystring': querystring,
            'active': search_type,
            'count': count,
            'results': results[search_type],
        })
    else:
        return render(request, 'search/search.html', {'hide_search': True})
